@@ -1,4 +1,3 @@
-{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 -- |
 -- Module:      Data.Aeson
@@ -77,6 +76,7 @@ module Data.Aeson
     , fromJSON
     , ToJSON(..)
     , KeyValue(..)
+    , KeyValueOmit(..)
     , (<?>)
     , JSONPath
     -- ** Keys for maps
@@ -92,14 +92,18 @@ module Data.Aeson
     -- ** Liftings to unary and binary type constructors
     , FromJSON1(..)
     , parseJSON1
+    , omittedField1
     , FromJSON2(..)
     , parseJSON2
+    , omittedField2
     , ToJSON1(..)
     , toJSON1
     , toEncoding1
+    , omitField1
     , ToJSON2(..)
     , toJSON2
     , toEncoding2
+    , omitField2
     -- ** Generic JSON classes and options
     , GFromJSON
     , FromArgs
@@ -124,6 +128,7 @@ module Data.Aeson
     , constructorTagModifier
     , allNullaryToStringTag
     , omitNothingFields
+    , allowOmittedFields
     , sumEncoding
     , unwrapUnaryRecords
     , tagSingleConstructors
@@ -152,24 +157,20 @@ module Data.Aeson
     , (.:?)
     , (.:!)
     , (.!=)
+    , (.:?=)
+    , (.:!=)
     , object
     -- * Parsing
-    , json
-    , json'
     , parseIndexedJSON
     ) where
 
-import Prelude.Compat
-
-import Control.Exception (Exception (..))
 import Control.Monad.Catch (MonadThrow (..))
-import Data.Aeson.Types.FromJSON (ifromJSON, parseIndexedJSON)
+import Data.Aeson.Types.FromJSON (parseIndexedJSON)
 import Data.Aeson.Encoding (encodingToLazyByteString)
-import Data.Aeson.Parser.Internal (decodeWith, decodeStrictWith, eitherDecodeWith, eitherDecodeStrictWith, jsonEOF, json, jsonEOF', json')
 import Data.Aeson.Types
-import Data.Aeson.Types.Internal (formatError)
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as L
+import Data.Aeson.Decoding (decode, eitherDecode, throwDecode, decodeStrict, eitherDecodeStrict, throwDecodeStrict)
 
 -- $setup
 -- >>> :set -XOverloadedStrings
@@ -184,32 +185,6 @@ encode = encodingToLazyByteString . toEncoding
 encodeFile :: (ToJSON a) => FilePath -> a -> IO ()
 encodeFile fp = L.writeFile fp . encode
 
--- | Efficiently deserialize a JSON value from a lazy 'L.ByteString'.
--- If this fails due to incomplete or invalid input, 'Nothing' is
--- returned.
---
--- The input must consist solely of a JSON document, with no trailing
--- data except for whitespace.
---
--- This function parses immediately, but defers conversion.  See
--- 'json' for details.
-decode :: (FromJSON a) => L.ByteString -> Maybe a
-decode = decodeWith jsonEOF fromJSON
-{-# INLINE decode #-}
-
--- | Efficiently deserialize a JSON value from a strict 'B.ByteString'.
--- If this fails due to incomplete or invalid input, 'Nothing' is
--- returned.
---
--- The input must consist solely of a JSON document, with no trailing
--- data except for whitespace.
---
--- This function parses immediately, but defers conversion.  See
--- 'json' for details.
-decodeStrict :: (FromJSON a) => B.ByteString -> Maybe a
-decodeStrict = decodeStrictWith jsonEOF fromJSON
-{-# INLINE decodeStrict #-}
-
 -- | Efficiently deserialize a JSON value from a file.
 -- If this fails due to incomplete or invalid input, 'Nothing' is
 -- returned.
@@ -226,54 +201,30 @@ decodeFileStrict = fmap decodeStrict . B.readFile
 -- If this fails due to incomplete or invalid input, 'Nothing' is
 -- returned.
 --
--- The input must consist solely of a JSON document, with no trailing
--- data except for whitespace.
+-- Since @2.2.0.0@ an alias for 'decode'.
 --
--- This function parses and performs conversion immediately.  See
--- 'json'' for details.
 decode' :: (FromJSON a) => L.ByteString -> Maybe a
-decode' = decodeWith jsonEOF' fromJSON
+decode' = decode
 {-# INLINE decode' #-}
 
 -- | Efficiently deserialize a JSON value from a strict 'B.ByteString'.
 -- If this fails due to incomplete or invalid input, 'Nothing' is
 -- returned.
 --
--- The input must consist solely of a JSON document, with no trailing
--- data except for whitespace.
+-- Since @2.2.0.0@ an alias for 'decodeStrict'.
 --
--- This function parses and performs conversion immediately.  See
--- 'json'' for details.
 decodeStrict' :: (FromJSON a) => B.ByteString -> Maybe a
-decodeStrict' = decodeStrictWith jsonEOF' fromJSON
+decodeStrict' = decodeStrict
 {-# INLINE decodeStrict' #-}
 
 -- | Efficiently deserialize a JSON value from a file.
 -- If this fails due to incomplete or invalid input, 'Nothing' is
 -- returned.
 --
--- The input file's content must consist solely of a JSON document,
--- with no trailing data except for whitespace.
+-- Since @2.2.0.0@ an alias for 'decodeFileStrict'.
 --
--- This function parses and performs conversion immediately.  See
--- 'json'' for details.
 decodeFileStrict' :: (FromJSON a) => FilePath -> IO (Maybe a)
-decodeFileStrict' = fmap decodeStrict' . B.readFile
-
-eitherFormatError :: Either (JSONPath, String) a -> Either String a
-eitherFormatError = either (Left . uncurry formatError) Right
-{-# INLINE eitherFormatError #-}
-
--- | Like 'decode' but returns an error message when decoding fails.
-eitherDecode :: (FromJSON a) => L.ByteString -> Either String a
-eitherDecode = eitherFormatError . eitherDecodeWith jsonEOF ifromJSON
-{-# INLINE eitherDecode #-}
-
--- | Like 'decodeStrict' but returns an error message when decoding fails.
-eitherDecodeStrict :: (FromJSON a) => B.ByteString -> Either String a
-eitherDecodeStrict =
-  eitherFormatError . eitherDecodeStrictWith jsonEOF ifromJSON
-{-# INLINE eitherDecodeStrict #-}
+decodeFileStrict' = decodeFileStrict
 
 -- | Like 'decodeFileStrict' but returns an error message when decoding fails.
 eitherDecodeFileStrict :: (FromJSON a) => FilePath -> IO (Either String a)
@@ -282,78 +233,46 @@ eitherDecodeFileStrict =
 {-# INLINE eitherDecodeFileStrict #-}
 
 -- | Like 'decode'' but returns an error message when decoding fails.
+--
+-- Since @2.2.0.0@ an alias for 'eitherDecode'.
 eitherDecode' :: (FromJSON a) => L.ByteString -> Either String a
-eitherDecode' = eitherFormatError . eitherDecodeWith jsonEOF' ifromJSON
+eitherDecode' = eitherDecode
 {-# INLINE eitherDecode' #-}
 
 -- | Like 'decodeStrict'' but returns an error message when decoding fails.
+--
+-- Since @2.2.0.0@ an alias for 'eitherDecodeStrict'.
 eitherDecodeStrict' :: (FromJSON a) => B.ByteString -> Either String a
-eitherDecodeStrict' =
-  eitherFormatError . eitherDecodeStrictWith jsonEOF' ifromJSON
+eitherDecodeStrict' = eitherDecodeStrict
 {-# INLINE eitherDecodeStrict' #-}
 
 -- | Like 'decodeFileStrict'' but returns an error message when decoding fails.
+--
+-- Since @2.2.0.0@ an alias for 'eitherDecodeFileStrict''.
 eitherDecodeFileStrict' :: (FromJSON a) => FilePath -> IO (Either String a)
-eitherDecodeFileStrict' =
-  fmap eitherDecodeStrict' . B.readFile
+eitherDecodeFileStrict' = eitherDecodeFileStrict
 {-# INLINE eitherDecodeFileStrict' #-}
 
-throwFormatError :: MonadThrow m => Either (JSONPath, String) a -> m a
-throwFormatError = either (throwM . AesonException . uncurry formatError) return
-{-# INLINE throwFormatError #-}
-
--- | Like 'decode' but throws an 'AesonException' when decoding fails.
---
--- >>> throwDecode "42" :: Maybe Int
--- Just 42
---
--- >>> throwDecode "42" :: IO Int
--- 42
---
--- >>> throwDecode "true" :: IO Int
--- ...Exception: AesonException...
---
--- @since 2.1.2.0
---
-throwDecode :: forall a m. (FromJSON a, MonadThrow m) => L.ByteString -> m a
-throwDecode = throwFormatError . eitherDecodeWith jsonEOF ifromJSON
-{-# INLINE throwDecode #-}
-
--- | Like 'decodeStrict' but throws an 'AesonException' when decoding fails.
---
--- @since 2.1.2.0
---
-throwDecodeStrict :: forall a m. (FromJSON a, MonadThrow m) => B.ByteString -> m a
-throwDecodeStrict =
-  throwFormatError . eitherDecodeStrictWith jsonEOF ifromJSON
-{-# INLINE throwDecodeStrict #-}
-
 -- | Like 'decode'' but throws an 'AesonException' when decoding fails.
 --
+-- Since @2.2.0.0@ an alias for 'throwDecode'.
+--
 -- @since 2.1.2.0
 --
 throwDecode' :: forall a m. (FromJSON a, MonadThrow m) => L.ByteString -> m a
-throwDecode' = throwFormatError . eitherDecodeWith jsonEOF' ifromJSON
+throwDecode' = throwDecode
 {-# INLINE throwDecode' #-}
 
 -- | Like 'decodeStrict'' but throws an 'AesonException' when decoding fails.
 --
+-- Since @2.2.0.0@ an alias for 'throwDecodeStrict'.
+--
 -- @since 2.1.2.0
 --
 throwDecodeStrict' :: forall a m. (FromJSON a, MonadThrow m) => B.ByteString -> m a
-throwDecodeStrict' =
-  throwFormatError . eitherDecodeStrictWith jsonEOF' ifromJSON
+throwDecodeStrict' = throwDecodeStrict
 {-# INLINE throwDecodeStrict' #-}
 
--- | Exception thrown by 'throwDecode' and variants.
---
--- @since 2.1.2.0
-newtype AesonException = AesonException String
-  deriving (Show)
-
-instance Exception AesonException where
-    displayException (AesonException str) = "aeson: " ++ str
-
 -- $use
 --
 -- This section contains basic information on the different ways to
